Package: coRdon
Type: Package
Title: Codon Usage Analysis and Prediction of Gene Expressivity
Version: 1.31.0
Authors@R: 
    c(person(given = "Anamaria",
             family = "Elek",
             role = c("cre", "aut"),
             email = "anamariaelek@gmail.com"),
      person(given = "Maja",
             family = "Kuzman",
             role = "aut",
             email = "mkuzman@mail.bioinfo.hr"),
      person(given = "Kristian",
             family = "Vlahovicek",
             role = "aut",
             email = "kristian@bioinfo.hr"))
Description: Tool for analysis of codon usage in various 
    unannotated or KEGG/COG annotated DNA sequences. 
	Calculates different measures of CU bias and CU-based
	predictors of gene expressivity, and performs gene set 
	enrichment analysis for annotated sequences.
	Implements several methods for visualization of CU
	and enrichment analysis results.
License: Artistic-2.0
LazyData: TRUE
biocViews: Software, Metagenomics, GeneExpression, GeneSetEnrichment,
        GenePrediction, Visualization, KEGG, Pathways, Genetics
        CellBiology, BiomedicalInformatics, ImmunoOncology
Depends: R (>= 3.5)
Imports: methods, stats, utils, Biostrings, Biobase, dplyr, stringr,
        purrr, ggplot2, data.table
Suggests: BiocStyle, testthat, knitr, rmarkdown
RoxygenNote: 6.1.1
Collate: 'coRdon.R' 'genCode-class.R' 'codonTable-class.R'
        'functions.R' 'codonUsage.R' 'codonUsage-expressivity.R'
        'codonUsage-visualization.R' 'crossTab-class.R' 'data.R'
        'enrichment-visualization.R' 'enrichment.R' 'readSet.R'
VignetteBuilder: knitr
URL: https://github.com/BioinfoHR/coRdon
BugReports: https://github.com/BioinfoHR/coRdon/issues
git_url: https://git.bioconductor.org/packages/coRdon
git_branch: devel
git_last_commit: 86ceb2e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:59:50 UTC; biocbuild
Author: Anamaria Elek [cre, aut],
  Maja Kuzman [aut],
  Kristian Vlahovicek [aut]
Maintainer: Anamaria Elek <anamariaelek@gmail.com>
