% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{LD94}
\alias{LD94}
\title{Codon usage in human gut microbiome in liver cirrhosis.}
\format{A \code{codonTable} object.}
\source{
\href{https://go.nature.com/2G7QdpC}{Quin et al. 2014};
\href{https://bit.ly/2DRfiz6}{Fabijanic and Vlahovicek 2016}
}
\usage{
LD94
}
\description{
A \code{codonTable} object with codon counts for sequences of the human gut
metagenome, from an individual with liver cirrhosis. Raw sequences are from
\href{https://go.nature.com/2G7QdpC}{Quin et al. 2014},
processed, assembled and annotated (KEGG Orthology) as descrbed in
\href{https://bit.ly/2DRfiz6}{Fabijanic and Vlahovicek 2016}.
Due to size limitations, only a sample of 1000 sequences from the original
data is used.
}
\keyword{datasets}
